/*
     CMTZMAN_LIB
     Copyright (C) 2005-6 Peter Briggs

     This code is distributed under the terms and conditions of the
     CCP4 Program Suite Licence Agreement as a CCP4 Application.
     A copy of the CCP4 licence can be obtained by writing to the
     CCP4 Secretary, Daresbury Laboratory, Warrington WA4 4AD, UK
*/

/* cmtzman_lib.h

   Header for pseudo-library for CMTZMAN program

*/
#include <math.h>
#include "cmtzlib.h"
#include "ccp4_general.h"
#include "ccp4_array.h"

/*********************************************
 * Hashing of MTZ reflection data
 *********************************************/

/* Parameters for the hash table */
enum {
  MULTIPLIER = 37,
  NHASH = 6151
};

/* Linked list element for the hash table */
typedef struct MtzHashedRefl MtzHashedRefl;
struct MtzHashedRefl {
  int h;
  int k;
  int l;
  int index;
  MtzHashedRefl *next;
};

/* Hash table structure */
typedef struct MtzReflHashTable MtzReflHashTable;
struct MtzReflHashTable {
  /* Hash table parameters */
  MtzHashedRefl **hashtable;
};

MtzReflHashTable* InitMtzReflHashTable(MTZ *mtz);

int LookupMtzRefl(MtzReflHashTable *refltab, int ihkl[3], int insert, int index);

int MTZhash(int ihkl[3]);

int FreeMtzReflHashTable(MtzReflHashTable* refltab);

/*********************************************
 * MTZ-related functions
 *********************************************/

int merge_mtzs(MTZ *mtz1, MTZ *mtz2);

MTZCOL* move_column(MTZ *mtz, MTZCOL *col, MTZSET *new_set);

MTZXTAL *new_crystal(MTZ *mtz, char *xname, float cell[]);

MTZSET *new_dataset(MTZ *mtz, char *dname, MTZXTAL *xtal,
		    float wavelength);

MTZSET *move_dataset(MTZ *mtz, MTZSET *set, MTZXTAL *new_xtal);

int rename_attribute(MTZ *mtz, char *attribute, char *old,
		     char *new, char *parent);

int validate_names(MTZ *mtz);

int fix_name(char *name, char *fixed_name);

int extract_names(char *fullname, char *xname, char *dname);

int show_hierarchy(MTZ *mtz);

/****************************************************************
  MTZ UTILITIES

  These are utility functions which should be considered for
  migration into the CMTZ library.
*****************************************************************/

char *MtzXtalName(MTZXTAL *xtal);

int MtzSetXtalCell(MTZXTAL *xtal, float cell[]);

char *MtzSetProjectName(MTZXTAL *xtal, char *pname);

char *MtzDatasetName(MTZSET *set);

float MtzDatasetWavelength(MTZSET *set);

int MtzSetDatasetWavelength(MTZSET *set, float wavelength);

char *MtzSetXtalName(MTZXTAL *xtal, char *xname);

char *MtzSetDatasetName(MTZSET *set, char *dname);

char *MtzSetColName(MTZCOL *col, char *label);

int MtzSetColType(MTZCOL *col, char *ctype);

int MtzAddRefln(MTZ *mtz, int ihkl[]);

int MtzDeleteXtal(MTZ *mtz, MTZXTAL *xtal);

int MtzDeleteDataset(MTZ *mtz, MTZSET *set);

int MtzDeleteColumn(MTZ *mtz, MTZCOL *col);

int GetHKLFromI(MTZCOL *hkl[], int i, int ihkl[]);

int GetIFromHKL(MTZ *mtz, MTZCOL *hkl[], int ihkl[]);

/****************************************************************
 * Supplementary functions
 * Some of these are shared with other programs that I've written
 ****************************************************************/

int WriteLogicalName(FILE *fp, char *logicalname);

int fix_name(char *name, char *fixed_name);

int extract_names(char *fullname, char *xname, char *dname);

